#include ecos:shaders/pass/header.glsl
#include ecos:options

// FEATURE: sun & moon bloom
// #define FEAT_SUN_MOON_BLOOM
#if defined(BLOOM) && defined(FEAT_SUN_MOON_BLOOM)
#include frex:shaders/api/view.glsl
#include frex:shaders/api/world.glsl
#endif

uniform sampler2D u_main_color;
uniform sampler2D u_main_depth;
uniform sampler2D u_translucent_color;
uniform sampler2D u_translucent_depth;
uniform sampler2D u_entity_color;
uniform sampler2D u_entity_depth;
uniform sampler2D u_weather_color;
uniform sampler2D u_weather_depth;
uniform sampler2D u_clouds_color;
uniform sampler2D u_clouds_depth;
uniform sampler2D u_particles_color;
uniform sampler2D u_particles_depth;

in vec2 texcoord;

out vec4 fragColor;

vec4[6] color_values;
float[6] depth_values;
int len = 0;

void insert_sort(vec4 color, float depth)
{
  // Filter out fully transparent pixel
  if (color.a == 0.0) {
    return;
  }

  color_values[len] = color;
  depth_values[len] = depth;
  len ++;
  
  int seek = len - 1;
  int before = seek - 1;

  while (seek > 0 && depth_values[seek] > depth_values[before]) {
    vec4 temp_color = color_values[seek];
    float temp_depth = depth_values[seek];

    color_values[seek] = color_values[before];
    depth_values[seek] = depth_values[before];
    color_values[before] = temp_color;
    depth_values[before] = temp_depth;

    seek = before;
    before --;
  }
}

vec3 blend_colors(vec3 destination, vec4 source)
{
  return source.rgb + destination * (1.0 - source.a);
}

void main()
{
  vec4  main_color        = texture(u_main_color       , texcoord);
  float main_depth        = texture(u_main_depth       , texcoord).r;
  vec4  translucent_color = texture(u_translucent_color, texcoord);
  float translucent_depth = texture(u_translucent_depth, texcoord).r;
  vec4  entity_color      = texture(u_entity_color     , texcoord);
  float entity_depth      = texture(u_entity_depth     , texcoord).r;
  vec4  weather_color     = texture(u_weather_color    , texcoord);
  float weather_depth     = texture(u_weather_depth    , texcoord).r;
  vec4  clouds_color      = texture(u_clouds_color     , texcoord);
  float clouds_depth      = texture(u_clouds_depth     , texcoord).r;
  vec4  particles_color   = texture(u_particles_color  , texcoord);
  float particles_depth   = texture(u_particles_depth  , texcoord).r;

  // Guarantee main layer to be inserted
  color_values[0] = vec4(main_color.rgb, 1.0); // Solid layer might have gibberish alpha
  depth_values[0] = main_depth;
  len = 1;

  insert_sort(translucent_color, translucent_depth);
  insert_sort(entity_color, entity_depth);
  insert_sort(particles_color, particles_depth);
  insert_sort(weather_color, weather_depth);
  insert_sort(clouds_color, clouds_depth);
  
  vec3 composite_color = color_values[0].rgb;

  for(int i=1; i < len; i++){
    composite_color = blend_colors(composite_color, color_values[i]);
  }
  
  fragColor.rgb = composite_color;
  
  #if defined(BLOOM) && defined(FEAT_SUN_MOON_BLOOM)
    float final_depth = depth_values[len - 1];
    vec4 pos = vec4(texcoord * 2.0 - 1.0, final_depth * 2.0 - 1.0, 1.0);
    pos = frx_inverseViewProjectionMatrix * pos;
    pos.xyz /= pos.w;
    fragColor.a = pow(max(0.0, dot(normalize(pos.xyz), frx_skyLightVector)), 100.0) * step(1.0, final_depth);
  #endif
}
